% Run xml_processing.m in MATLAB 2016 to export Transition.mat, Event_ID.mat, Marking_state_ID.mat from Autopilot.xml.
% The file Autopilot.xml is a file containing the supervisor information, exported from Supremica software.
% The file xml_processing.m is a program processing the Autopilot.xml to generate three matrices: Transition, Event_ID, Marking_state_ID.
% The ouput fileTransition.mat: an m*3 matrix, where m is the number of transitions in the synthesized supervisor. In each row, it consists of a source state, a destination state and a triggered event.
% The output file Marking_state_ID.mat: a 1*8 matrix, where 8 is the number of the marking states, including POWER OFF MODE, STANDBY MODE, GROUND-ERROR MODE, LOITER MODE, ALTITUDE-HOLD MODE, STABILIZE MODE, RTL MODE and AL MODE. This matrix stores the ID of the marking states in the supervisor.

clear;  
xDoc = xmlread('Autopilot.xml');     
xRoot=xDoc.getDocumentElement();
Events= xRoot.getElementsByTagName('Event');
Num_Events=Events.getLength();
Event=cell(Num_Events,3);
States= xRoot.getElementsByTagName('State');
Num_States=States.getLength();
State=cell(Num_States,3);
Transitions= xRoot.getElementsByTagName('Transition');
Num_Transitions=Transitions.getLength();
Transition=cell(Num_Transitions,3);
Event1=Events.item(0);
State1=States.item(0);

Transition1=Transitions.item(0);
for i=1:Num_Events
    Event1id=char(Event1.getAttribute('id'));
    Event1label=char(Event1.getAttribute('label'));
    Event1controllable=char(Event1.getAttribute('controllable'));
    Event{i,1}=Event1id;
    Event{i,2}=Event1label;
    Event{i,3}=Event1controllable;
Event1=Events.item(i);
end

for i=1:Num_States
    State1id=char(State1.getAttribute('id'));
    State1name=char(State1.getAttribute('name'));
    State1accepting=char(State1.getAttribute('accepting'));
    State{i,1}=State1id;
    State{i,2}=State1name;
    State{i,3}=State1accepting;
State1=States.item(i);
end

for i=1:Num_Transitions
    Transition1source=char(Transition1.getAttribute('source'));
    Transition1dest=char(Transition1.getAttribute('dest'));
    Transition1event=char(Transition1.getAttribute('event'));
    Transition{i,1}=Transition1source;
    Transition{i,2}=Transition1dest;
    Transition{i,3}=Transition1event;
Transition1=Transitions.item(i);
end

[m1,n1]=size(Transition);
for i=1:m1
for j=1:n1
Transition{i,j}=str2double(Transition{i,j});
end
end

Transitiontable=cell2mat(Transition);
[m2,n2]=size(State);
j=1;
for i=1:m2
    if strcmp(State(i,3),'true')==1
        Statetable(j)=State(i,1);
        j=j+1;
    end
end
[m1,n1]=size(Statetable);
for i=1:m1
for j=1:n1
Statetable{i,j}=str2double(Statetable{i,j});
end
end
Statetable=cell2mat(Statetable);
% end

%% MCEEvent
[m1,n1]=size(Transitiontable);
[m2,n2]=size(Statetable);
l=1;
for i=1:m1
    for j=1:n2
        if Transitiontable(i,2)==Statetable(1,j)
            MCEtables(l,:)=Transitiontable(i,:);
            l=l+1;
            break;
        end
    end
end

%% outMCE

Event1=Event(:,2);
[Eventr,Eventc]=size(Event1);
j=1;
for i=1:Eventr
    tex=strfind(Event1(i),'MCE');
    if tex{1}==1
        MCEs(j)=Event1(i);
        j=j+1;
    end
end


%% MCEEvent
Event1=Event(:,2);
[m1,n1]=size(Event1);
[m2,n2]=size(MCEs);
l=1;
for i=1:n2
    for j=1:m1
    if strcmp(Event1(j),MCEs(i))==1
       MCElist(l)=j-1;
       l=l+1;
       break;
    end
    end
end

%% outstatetable
[MCEtablesr,MCEtablesc]=size(MCEtables);
for i=1:MCEtablesr
    for j=1:MCEtablesr
        if MCEtables(j,3)==MCElist(i)
            Statetable(i)=MCEtables(j,2);
            break;
        end
    end
end

Transition=Transitiontable;
Event_ID=Event;
Marking_state_ID=Statetable;
save('Transition.mat','Transition');
save('Event_ID.mat', 'Event_ID');
save('Marking_state_ID', 'Marking_state_ID');
